#!/usr/bin/perl -w
#annotation is based on configuration file and more rules
#Some constraint rules are implemented as perl functions in rules.pm
#         rules: 1. One CPU will have at least 64M memory.
#                2. One CPU will have at most 4G memory.
#                3. More CPU, more likely to have bigger memory.
#                4. One CPU will have at least 10G disk.
#                5. More CPU, more likely to have bigger disk.
#                6. More memory, more likely to have bigger disk.
#                7. More disk, more likely to have more memory.
#                8. Intel&Windows box will have at most 4 CPUs, other type may have up to 1024 CPUs.
#                9. Intel Arch only has Windows and LINUX as OS.
#                10. Intel Arch and other Arch have different distributions of CPU MHZ.
#                11. Different OS has its own vendor. 
#                12. Host load is purely random.

use standard_format;
use rules;



sub SetupHashes 
{
    my ($line, $hashName, @keys, @values, $len, $hashlength, @TEMP);
    open (DistConf, "./distributation.conf") or die "Can't open config file distributation.conf: $!";

    while($line = <DistConf>) {
	if(($line =~ /^\s+\b/) or ($line =~ /^#/)) {
	    ; #do nothing
	}else{
	    chomp $line;
	    @TEMP = split(/\s+/, $line);
	    if($TEMP[0] =~ /RATIO-\w+/)
	    {
		@keys = @TEMP;
		$hashName = $keys[0];
		#push (@hashes, $hashName);
		$hashlength = @keys;
		
		for($len=2; $len<$hashlength; $len++)
		{
		    $keys[$len] = $keys[$len] + $keys[$len-1];
		}

	    }elsif($TEMP[0] =~ /ALPHA/){
		$ALPHA = $TEMP[1];
	    }elsif($TEMP[0] =~ /POWER/){
		$POWER = $TEMP[1];
	    }elsif($TEMP[0] =~ /^\s+\b/){
		#do nothing if blank line
		;
	    }elsif (($TEMP[0] !~ /RATIO-\w+/) && ($TEMP[0] !~ /#\w+/)) {
		  @values = @TEMP;
		  for($len=1; $len < $hashlength; $len++)
		     {
			 ${$hashName}{$keys[$len]} = $values[$len];
		     }
		} #elsif 
	} #end of else
       } #while
 }
    

sub SetupPowerlaw {
    my $i = 2;
    my ($cur, $CurSum);

    $cur = ((2.7183**$ALPHA)*($i**$POWER));
    $CurSum = $cur;
    $POWERLAW{$cur} = 1;
    do {
	$cur = (2.7183**$ALPHA)*($i**$POWER);
	$CurSum = $CurSum + $cur;
	$POWERLAW{$CurSum} = $i;
	$i++;
    }while($cur > 1);
    $ALLPOWER = $CurSum;
}


sub match {
    my ($myhash, $myseed) = @_;
    my (@temp, $val);
    @temp = sort {$a <=> $b} keys(%{$myhash});
    foreach $val (@temp)
    {
	if($myseed < $val)
	{
	    #print "seed is : $myseed, val is : $val, return val is : ${$myhash}{$val} \n"; 
	    return ${$myhash}{$val}; 
	}
    }

    print STDERR "Match failed! probablely wrong.\n";
    return -1;
}



#global variables
my $switchcap;
my $outdegree;
my $numcpu;
my $cpuspeed;
my $mem;
my $disk;
my $arch;
my $OS;
my $osvendor;
my $load;
my $type;
my $nodenum;
my $x;
my $y;
my $nettype;
my $hostperlan = 10;

my $totalnumrouter = 0;
my $totalnumhost = 0;
my $totalnumlink = 0;


#main starts here

  $OSconcentration = ifOSconcentration();

  $hostperlan = shift;
  $hostcounter = 1;

  $infile = STDIN;
  $outfile = STDOUT;
  open(NumEdgeNode, ">numedgenode.dat") or die "Can't open file $!\n";
  SetupHashes();
  SetupPowerlaw();
  srand(time);

#Annotate the nodes
  $cleanline=skip_comments($infile);

  do {
    #random seed for osvendor generation
    $osvendorseed = rand100();

    #random seed for load generation
    $loadseed = rand100();

    #random seed for router info generation
    $routerseed = rand100();

    #random seed for arch info generation
    $archseed = rand100();

    #random seed for os info generation
    $osseed = rand100();

    #random seed for numcpu info generation
    $numcpuseed = rand100();

    #random seed for cpuspeed info generation
    $cpuspeedseed = rand100();

    #random seed for mem info generation
    $memseed = rand100();

    #random seed for disk info generation
    $diskseed = rand100();

    #rand seed for promotion probability 
    $ppseed = rand100();

    #rand seed for degrade probility
    $dpseed = rand100();

    #rand seed for powerlaw
    $powerseed = randGen(1, $ALLPOWER);

    $concenseed = rand100();
    $dominArchseed = rand100();
    $dominOSseed = rand100();

    #print "$powerseed, $ALLPOWER, \n\n";
    chomp($cleanline);
    ($nodenum,$x,$y,$nettype) = split(/\s+/,$cleanline);
    if ($nettype==0) {
      $type=qw(ROUTER);
      $switchcap = match(qw(RATIO-WAN-SWITCH-CAP), $routerseed);
      $outdegree = match(qw(POWERLAW),$powerseed);  
      push @nodesp, [$type,$nodenum,$x,$y,$nettype,$switchcap,$outdegree];
      print $outfile "$type $nodenum $x $y $nettype $switchcap $outdegree\n";
      $totalnumrouter++;
    } elsif ($nettype == 1) { 
	$type=qw(ROUTER);
       	$switchcap = match(qw(RATIO-MAN-SWITCH-CAP), $routerseed);
        $outdegree = match(qw(POWERLAW), $powerseed);  
	push @nodesp, [$type,$nodenum,$x,$y,$nettype,$switchcap,$outdegree];
	print $outfile "$type $nodenum $x $y $nettype $switchcap $outdegree\n";
	$totalnumrouter++;
    } elsif ($nettype == 2) {
	$type = qw(HOST);
	$IfSatisfyOtherRule = 1;

	if($OSconcentration == 1) # if OS concentration is imposed 
	{
	    if($hostcounter % $hostperlan == 1) # for every subnet
	    {
		$concentrate = match(qw(RATIO-SUBNET-CONCENTRATION), $concenseed);
		$dominateArch = match(qw(RATIO-ARCH-CONCENTRATION), $dominArchseed);
		if($dominateArch eq qw(IA32))
		{
		    $dominateOS = match(qw(RATIO-IA-OSCONCENTRATION), $dominOSseed);
		    #print STDERR "$dominateOS \t";
		} elsif ($dominateArch eq qw(SPARC32))
		{
		    $dominateOS = match(qw(RATIO-SPARC-OSCONCENTRATION), $dominOSseed);
		    #print STDERR "$dominateOS \t";
		} elsif ($dominateArch eq qw(PPC))
		{
		    $dominateOS = match(qw(RATIO-PPC-OSCONCENTRATION), $dominOSseed);
		    #print STDERR "$dominateOS \t";
		}
	    }
	}
	
	$ifchangearch = 0;

	$arch = match(qw(RATIO-HOST-ARCH), $archseed);
	$archDominSeed = rand100();
	
	if($OSconcentration == 1) # if OS concentration is imposed 
	{
	    if($archDominSeed < $concentrate)
	    {
		$arch = $dominateArch;
		$ifchangearch = 1;
	    }
	}
	do { #this "do while" make sure that other user defined rules are satisfied
	    do {
		if(($arch eq "IA32") or ($arch eq "IA64")) 
		{
		    $OS = match(qw(RATIO-HOST-IA-OS), $osseed);
		    $numcpu = match(qw(RATIO-HOST-IA-OS-NUMCPU), $numcpuseed);
		    $cpuspeed = match(qw(RATIO-HOST-IA-OS-CPUMHZ), $cpuspeedseed);
		}else{   #match OS for other archs
		    $OS = match(qw(RATIO-HOST-OTHERARCH-OS), $osseed);
		    $numcpu = match(qw(RATIO-HOST-OTHERARCH-OS-NUMCPU), $numcpuseed);
		    $cpuspeed = match(qw(RATIO-HOST-OTHERARCH-OS-CPUMHZ), $cpuspeedseed);
		}

		if($OSconcentration == 1) # if OS concentration is imposed 
		{
		    if($ifchangearch == 1) #if the arch has been changed to dominant arch
		    {
			$OS = $dominateOS; 
		    }
		}

		$ifOSCPUok = ARCHandOSandCPUrule($arch, $OS, $numcpu, $cpuspeed);
		if ($ifOSCPUok == 0) {
		    $osseed = rand100();
		    $numcpuseed = rand100();
		    $cpuspeedseed = rand100();
		}
	    }while($ifOSCPUok == 0);

	    
	    do {
		# $pp is the promotion probability of memory size and disk size
		# $dp is the degrade probability of memory size and disk size
		if($numcpu <= 4) 
		{
		    $mem = match(qw(RATIO-HOST-4CPU-MEM), $memseed);
		    $disk = match(qw(RATIO-HOST-4CPU-DISK), $diskseed);
		    
		    $pp = promotionProb($numcpu, 4, 1);
		    $dp = degradeProb($numcpu, 4, 1);
	
		    if($ppseed <= $pp){		   
			$mem = promotionMemRate($numcpu, 4, 1, $mem);
			$disk = promotionDiskRate($numcpu, 4, 1, $disk);
		    }
		    if($dpseed <= $dp){			
			$mem = degradeMemRate($numcpu, 4, 1, $mem);
			$disk = degradeDiskRate($numcpu, 4, 1, $disk);
		    }
		    
		}elsif ($numcpu <= 32){	    
		    
		    $mem = match(qw(RATIO-HOST-32CPU-MEM), $memseed);	
		    $disk = match(qw(RATIO-HOST-32CPU-DISK), $diskseed);
		    
		    $pp = promotionProb($numcpu, 32, 4);
		    $dp = degradeProb($numcpu, 32, 4);
		    
		    if($ppseed <= $pp){			
			$mem = promotionMemRate($numcpu, 32, 4, $mem);
			$disk = promotionDiskRate($numcpu, 32, 4, $disk);
		    }
		    if($dpseed <= $dp){
			$mem = degradeMemRate($numcpu, 32, 4, $mem);
			$disk = degradeDiskRate($numcpu, 32, 4, $disk);
		    }

		}elsif ($numcpu <= 128){
		    $mem = match(qw(RATIO-HOST-128CPU-MEM), $memseed);	    
		    $disk = match(qw(RATIO-HOST-128CPU-DISK), $diskseed);
		    
		    $pp = promotionProb($numcpu, 128, 32);
		    $dp = degradeProb($numcpu, 128, 32);

		    if($ppseed <= $pp){       
			$mem = promotionMemRate($numcpu, 128, 32, $mem);
			$disk = promotionDiskRate($numcpu, 128, 32, $disk);
		    }
		    if($dpseed <= $dp){
			$mem = degradeMemRate($numcpu, 128, 32, $mem);
			$disk = degradeDiskRate($numcpu, 128, 32, $disk);
		    }
		    
		}elsif ($numcpu <= 1024){	    
		    $mem = match(qw(RATIO-HOST-1024CPU-MEM), $memseed);	    
		    $disk = match(qw(RATIO-HOST-1024CPU-DISK), $diskseed);
		    
		    $pp = promotionProb($numcpu, 1024, 128);
		    $dp = degradeProb($numcpu, 1024, 128);

		    if($ppseed <= $pp){
			$mem = promotionMemRate($numcpu, 1024, 128, $mem);
			$disk = promotionDiskRate($numcpu, 1024, 128, $disk);
		    }
		    if($dpseed <= $dp){
			$mem = degradeMemRate($numcpu, 1024, 128, $mem);
			$disk = degradeDiskRate($numcpu, 1024, 128, $disk);
		    }
		}

		$ifMEMandDISKok = NUMCPUandMEMandDISKrule($arch, $OS, $numcpu, $cpuspeed, $mem, $disk);
		if($ifMEMandDISKok == 0){
		    $memseed = rand100();
		    $diskseed = rand100();
		}
	    } while ($ifMEMandDISKok == 0);
	    
	    do {
		if(($OS eq "WINXP") or ($OS eq "WIN2K") or ($OS eq "WINME") or 
		   ($OS eq "WIN98") or ($OS eq "WINNT") or ($OS eq "WIN95")) 
		{
		    $osvendor = match(qw(RATIO-HOST-WIN-OSVENDOR), $osvendorseed);
		}elsif ($OS eq "FreeBSD") {
		    $osvendor = match(qw(RATIO-HOST-FREEBSD-OSVENDOR), $osvendorseed);
		}elsif ($OS eq "LINUX") {
		    $osvendor = match(qw(RATIO-HOST-LINUX-OSVENDOR), $osvendorseed);
		}elsif ($OS eq "NetBSD") {
		    $osvendor = match(qw(RATIO-HOST-NETBSD-OSVENDOR), $osvendorseed);
		}elsif ($OS eq "SOLARIS") {
		    $osvendor = match(qw(RATIO-HOST-SOLARIS-OSVENDOR), $osvendorseed);
		}elsif ($OS eq "DUX") {
		    $osvendor = match(qw(RATIO-HOST-DUX-OSVENDOR), $osvendorseed);
		}elsif ($OS eq "HPUX") {
		    $osvendor = match(qw(RATIO-HOST-HPUX-OSVENDOR), $osvendorseed);
		}elsif ($OS eq "MACOS") {
		    $osvendor = match(qw(RATIO-HOST-MACOS-OSVENDOR), $osvendorseed);
		}
		
		$ifOSVENDORok = OSVENDORrule($OS, $osvendor);
		if($ifOSVENDORok == 0){
		    $osvendorseed = rand100();
		}
	    }while ($ifOSVENDORok == 0);
	    
	    do {
		$load = match(qw(RATIO-HOST-LOAD), $loadseed);	    
		$ifLOADok = LOADrule($type,$nodenum,$x,$y,$nettype,$numcpu,
				     $cpuspeed,$mem,$disk,$arch,$OS,$osvendor);
		if($ifLOADok == 0){
		  $loadseed = rand100();  
		} 
	    }while($ifLOADok == 0);

	    $IfSatisfyOtherRule = AllOtherRules($type,$nodenum,$x,$y,$nettype,$numcpu,
				      $cpuspeed,$mem,$disk,$arch,$OS,$osvendor,$load);
	}until ($IfSatisfyOtherRule == 1);
	print $outfile "$type $nodenum $x $y $nettype $numcpu $cpuspeed $mem $disk $arch $OS $osvendor $load\n";
	$totalnumhost++;
	#push @nodesp, [$type,$nodenum,$x,$y,$nettype,$numcpu,$cpuspeed,$mem,$disk,$arch,$OS,$osvendor,$load];
	$hostcounter++;
    } #end of nettype=2
$cleanline=<$infile>;
} while (!is_comment($cleanline));





#now to deal with edges

#first read in links created by tiers
  while ($cleanline=<$infile>) {
    if (!is_comment($cleanline)) {
      chomp($cleanline);
      ($from,$to,$delay,$bw,$ft,$tt,$state) = split(/\s+/,$cleanline);
      #push @edgesp,  ["LINK", $from, $to, $delay, $bw, $ft, $tt, $state];
      push @edgesp,  [$from, $to];
      print $outfile "LINK $from $to $delay $bw $ft $tt $state\n";
      $totalnumlink++;
    }
  }


#reduce the outdegree according to the tree created by tiers
  for $i (0 .. $#edgesp) {
      for $j (0 .. $#nodesp) {
	  if($nodesp[$j][1] == $edgesp[$i][0]) {
	      $nodesp[$j][6]--;
	  }
      }
  }


#add redundant links according to outdegree power law
  $notadd = 0;
  for $i (0 .. $#nodesp) {
    for $j (0 .. $#nodesp) {
	if($i != $j && $nodesp[$i][$#{$nodesp[$i]}] > 0 && $nodesp[$j][$#{$nodesp[$j]}] > 0) {
            #the for loop checks to see if the link exists already
	    for $k (0 .. $#edgesp){
		if($edgesp[$k][0]==$nodesp[$i][1] && $edgesp[$k][1]==$nodesp[$j][1]
		   || $edgesp[$k][0]==$nodesp[$j][1] && $edgesp[$k][1]==$nodesp[$i][1]){
		    $notadd = 1;
		}
	    }
	    if($notadd == 0){
		$nodesp[$i][$#{$nodesp[$i]}]--;
		$nodesp[$j][$#{$nodesp[$j]}]--;
		push @edgesp,  [$nodesp[$i][1], $nodesp[$j][1]];
		push @edgesp,  [$nodesp[$j][1], $nodesp[$i][1]];
		print $outfile "LINK $nodesp[$i][1] $nodesp[$j][1] 0.001 100 5 5 2\n";
		print $outfile "LINK $nodesp[$j][1] $nodesp[$i][1] 0.001 100 5 5 2\n";
		$totalnumlink+=2;
	    }
	    $notadd = 0;
	}
    }
  }


print NumEdgeNode "TOTALNUMROUTER $totalnumrouter\n";
print NumEdgeNode "TOTALNUMHOST $totalnumhost\n";
print NumEdgeNode "TOTALNUMLINK $totalnumlink\n";




