package standard_format;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(read_standard_format 
	     write_standard_format 
	     read_tiers_general_format 
	     write_dot_format 
	     is_comment 
	     skip_comments 
	     randGen 
	     rand100
	     );


sub read_standard_format {
  my ($infile,$nodeptr,$edgeptr)=@_;
  my ($numline, $line);
  my ($numnodes, $numedges, $numrouters,$numhosts);
  my @q;


  open(NumEdgeNode, "numedgenode.dat") or die "Can't open file $!\n";
  while($numline = <NumEdgeNode>)
  {
      if($numline=~/TOTALNUMROUTER\s+(\d+)/){ 
	  $numrouters=$1;
      }
      if($numline=~/TOTALNUMHOST\s+(\d+)/){ 
	  $numhosts=$1;
      }
      if($numline=~/TOTALNUMLINK\s+(\d+)/){ 
	  $numedges=$1;
      }    
  }
  close(NumEdgeNode);
  #print STDERR "$numrouters, $numhosts, $numedges\n";
  $numnodes = $numrouters + $numhosts;

  for ($i=0;$i<$numnodes;$i++) {
    $line=<$infile>;
    @q=split(/\s+/,$line);
    push @{$nodeptr}, [@q];
  }
  
  for ($i=0;$i<$numedges;$i++) {
    $line=<$infile>;
    @q=split(/\s+/,$line);
    push @{$edgeptr}, [@q];
  }
  return ($#{$nodeptr}+1,$#{$edgeptr}+1);
}

sub write_standard_format {
  my ($outfile,$nodeptr,$edgeptr)=@_;
  my $line;
  my ($numnodes, $numedges);
  my $ptr;

  $numnodes=$#{$nodeptr}+1;
  $numedges=$#{$edgeptr}+1;

  print $outfile "NUMNODES $numnodes\nNUMEDGES $numedges\n";

  foreach $ptr (@{$nodeptr}) {
    print $outfile join("\t",@{$ptr}), "\n";
  }

  foreach $ptr (@{$edgeptr}) {
    print $outfile join("\t",@{$ptr}), "\n";
  }

}


sub read_tiers_general_format {
  my ($infile,$nodeptr,$edgeptr) = @_;
  my $line;
  my ($nodenum, $x, $y, $nettype);
  my $type;
  my ($from, $to, $delay, $bw, $ft, $tt, $state);


  $line=skip_comments($infile);
  do {
    chomp($line);
    ($nodenum,$x,$y,$nettype) = split(/\s+/,$line);
    if ($nettype==0) {
      $type="ROUTER";
    } elsif ($nettype==1) { 
      $type="ROUTER";
    } elsif ($nettype==2) {
      $type="HOST";
    }
    push @{$nodeptr}, [$type,$nodenum,$x,$y,$nettype];
    $line=<$infile>;
  } while (!is_comment($line));

  while ($line=<$infile>) {
    if (!is_comment($line)) {
      chomp($line);
      ($from,$to,$delay,$bw,$ft,$tt,$state) = split(/\s+/,$line);
      push @{$edgeptr},  ["LINK", $from, $to, $delay, $bw, $ft, $tt, $state];
    }
  }
  return ($#{$nodeptr}+1,$#{$edgeptr}+1);
} 


sub write_dot_format {
  my ($outfile,$nodeptr,$edgeptr) = @_;
  my $line;
  my ($first,$second,$third);

  print $outfile "digraph network {\n";

  foreach $ptr (@{$nodeptr}) {
    $first=${@{$ptr}}[1];
    print $outfile "n$first\n";
  }
  foreach $ptr (@{$edgeptr}) {
    $second=${@{$ptr}}[1];
    $third=${@{$ptr}}[2];
    print $outfile "n$second -> n$third;\n";
  }
  print $outfile "}\n";
}



sub is_comment {
  return ($_[0] =~/^\#/);
}


sub skip_comments {
  my $file = shift;
  
  while ($line=<$file>) {
    if (!is_comment($line))  {
      return $line;
    }
  }
}


#////////////////////////////////////
#// generate a rand int number in between [lower, upper)

sub randGen {
    my ($lower, $upper) = @_;
    my $randnum = int(rand($upper-$lower) + $lower); 

    return $randnum;
}

sub rand100 {
    return randGen(1, 100);
}

